-- Test surroundings_ad
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(44);

set search_path to camper, public;

select has_table('surroundings_ad');
select has_pk('surroundings_ad');
select table_privs_are('surroundings_ad', 'guest', array['SELECT']);
select table_privs_are('surroundings_ad', 'employee', array['SELECT']);
select table_privs_are('surroundings_ad', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('surroundings_ad', 'authenticator', array[]::text[]);

select has_column('surroundings_ad', 'company_id');
select col_is_pk('surroundings_ad', 'company_id');
select col_is_fk('surroundings_ad', 'company_id');
select fk_ok('surroundings_ad', 'company_id', 'company', 'company_id');
select col_type_is('surroundings_ad', 'company_id', 'integer');
select col_not_null('surroundings_ad', 'company_id');
select col_hasnt_default('surroundings_ad', 'company_id');

select has_column('surroundings_ad', 'media_id');
select col_is_fk('surroundings_ad', 'media_id');
select fk_ok('surroundings_ad', 'media_id', 'media', 'media_id');
select col_type_is('surroundings_ad', 'media_id', 'integer');
select col_not_null('surroundings_ad', 'media_id');
select col_hasnt_default('surroundings_ad', 'media_id');

select has_column('surroundings_ad', 'title');
select col_type_is('surroundings_ad', 'title', 'text');
select col_not_null('surroundings_ad', 'title');
select col_hasnt_default('surroundings_ad', 'title');

select has_column('surroundings_ad', 'anchor');
select col_type_is('surroundings_ad', 'anchor', 'text');
select col_not_null('surroundings_ad', 'anchor');
select col_hasnt_default('surroundings_ad', 'anchor');

select has_column('surroundings_ad', 'href');
select col_type_is('surroundings_ad', 'href', 'uri');
select col_not_null('surroundings_ad', 'href');
select col_hasnt_default('surroundings_ad', 'href');


set client_min_messages to warning;
truncate surroundings_ad cascade;
truncate media cascade;
truncate media_content cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
     , (6, 'Company 5', 'XX345', '', '777-777-777', 'c@c', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into media_content (media_type, bytes)
values ('text/plain', 'content2')
     , ('text/plain', 'content4')
;

insert into media (media_id, company_id, original_filename, content_hash)
values ( 7, 2, 'text2.txt', sha256('content2'))
     , ( 8, 4, 'text3.txt', sha256('content4'))
;

insert into surroundings_ad (company_id, media_id, title, anchor, href)
values (2, 7, 'Ad 2', 'Go!', 'https://ddg.gg/')
     , (4, 8, 'Ad 4', 'Go!', 'https://ddg.gg/')
;

prepare surroundings_ad_data as
select company_id, title
from surroundings_ad
order by company_id, title;

set role guest;
select bag_eq(
	'surroundings_ad_data',
	$$ values (2, 'Ad 2')
	        , (4, 'Ad 4')
	$$,
	'Everyone should be able to list all surrounding ad across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ delete from surroundings_ad where company_id = 2 $$,
	'Admin from company 2 should be able to delete surrounding ad from that company.'
);

select bag_eq(
	'surroundings_ad_data',
	$$ values (4, 'Ad 4')
	$$,
	'The row should have been deleted.'
);

select lives_ok(
	$$ insert into surroundings_ad(company_id, media_id, title, anchor, href) values (2, 7, 'Another Ad', 'Go!', 'https://ddg.gg/') $$,
	'Admin from company 2 should be able to insert a new surrounding ad to that company.'
);

select bag_eq(
	'surroundings_ad_data',
	$$ values (2, 'Another Ad')
	        , (4, 'Ad 4')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update surroundings_ad set title = 'Ad 2' where company_id = 2 $$,
	'Admin from company 2 should be able to update surrounding ad of that company.'
);

select bag_eq(
	'surroundings_ad_data',
	$$ values (2, 'Ad 2')
	        , (4, 'Ad 4')
	$$,
	'The row should have been updated.'
);

select throws_ok(
	$$ insert into surroundings_ad (company_id, title) values (6, 'Ad 6') $$,
	'42501', 'new row violates row-level security policy for table "surroundings_ad"',
	'Admin from company 2 should NOT be able to insert new surrounding ad to company 6.'
);

select lives_ok(
	$$ update surroundings_ad set title = 'Nope' where company_id = 4 $$,
	'Admin from company 2 should not be able to update new surrounding ad of company 4, but no error if company_id is not changed.'
);

select bag_eq(
	'surroundings_ad_data',
	$$ values (2, 'Ad 2')
	        , (4, 'Ad 4')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update surroundings_ad set company_id = 6 where company_id = 2 $$,
	'42501', 'new row violates row-level security policy for table "surroundings_ad"',
	'Admin from company 2 should NOT be able to move surrounding ad to company 6'
);

select lives_ok(
	$$ delete from surroundings_ad where company_id = 4 $$,
	'Admin from company 2 should NOT be able to delete surrounding ad from company 4, but not error is thrown'
);

select bag_eq(
	'surroundings_ad_data',
	$$ values (2, 'Ad 2')
	        , (4, 'Ad 4')
	$$,
	'No row should have been changed'
);

reset role;


select *
from finish();

rollback;

